/*  g_ipc_opcje.c - command line parameters' parser
 *  
 *  (C) Copyright XII 2004 Michal 'GiM' Spadlinski <gim at skrzynka dot pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "g_ipc_opcje.h"

#define TEST_SYSV_QUEUE		0x01
#define TEST_SYSV_SHARED	0x02
#define TEST_SYSV_SHARED2	0x04
#define TEST_POSIX_QUEUE	0x08
#define TEST_POSIX_QUEUE2	0x10
#define TEST_POSIX_SHARED	0x20

/* 
 * I don't like getopt so I got here small parser :)
 */
g_opcja_t g_opcje[] = {
{ '1', "SystemV queue test",				NONE, 1, 0, 0 },
{ '2', "SystemV shared memory test 1",			NONE, 1, 0, 0 },
{ '3', "SystemV shared memory test 1",			NONE, 1, 0, 0 },
#ifdef HAS_POSIX_QUEUES
{ '4', "POSIX   queue test 1",				NONE, 1, 0, 0 },
{ '5', "POSIX   queue test 2",				NONE, 1, 0, 0 },
#endif
{ '6', "POSIX   shared memory test",			NONE, 1, 0, 0 },
{ 'a', "All tests",					NONE, 1, 0, 0 },
{ 'n', "number>0 of tests each test [default:10]",	INT,  1, 0, 0 },
{ 'h', "print this help message",			NONE, 1, 0, 0 }
};

/* 1 - help
 * 2 - allocation error
 * 0 - OK
 */
int g_ipc_skanuj_opcje(int *argc, char ***argv)
{
	int i;
	char **p=*argv, *q;
	p++;

	for (; *p ;)
	{
		if (**p == '-' && strlen(*p)>1)
		{
			for (i=0; i<G_END_OF_OPTS; i++)
				if ((*p)[1] == g_opcje[i].l) break;

			if (i == G_END_OF_OPTS)
				break;

			p++;
			if (g_opcje[i].type != NONE) {
				if (!g_opcje[i].argptr)
					g_opcje[i].argptr = *p;
				if (*p) p++;
			} else {
				if (g_opcje[i].val < g_opcje[i].maxocc)
					g_opcje[i].val++;
			}
		} else break;
	}

	/* if we have -h option any other parsing, building lists etc.
	 * is senseless, so let's get out of here
	 */
	if (g_opcje[G_HELP].val)
		return g_pokaz_pomoc((*argv)[0]);

	*argv = p;

	for (i=0; i<G_END_OF_OPTS; i++)
	{
		q = g_opcje[i].argptr;
		if (g_opcje[i].type == INT) {
			/* The only reasone for || is that 
			 * we don't want to make &NULL in call to g_atoi
			 */
			if (!q || g_atoi(&q, &(g_opcje[i].val)))
				g_opcje[i].argptr = NULL;
		}
	}

	return 0;
}

int g_atoi(char **buf, int *dest)
{
	char *x;
	long t;
	t = strtol(*buf, &x, 10);
	if (x == *buf) return 1;
	*dest = t;
	*buf = x;
	return 0;
}

int g_pokaz_pomoc(char *filename)
{
	int i;

fprintf(stderr, "\tSyntax: %s [options] (filename|pid)\nOptions:\n", filename);

	for (i=0; i<G_END_OF_OPTS; i++)
	{
		fprintf(stderr, "\t-%c\t%s\n", g_opcje[i].l, 
				g_opcje[i].desc);
	}

	return 1;
}

int g_pokaz_przekazane_opcje(void)
{
	char *q;
	int i;

fprintf(stderr, " running with following options:\n\t");
	for (i=0; i<G_END_OF_OPTS; i++)
	{
		q = g_opcje[i].argptr;
		if (g_opcje[i].val)
		{
			fprintf(stderr, "-%c ", g_opcje[i].l);
			if (g_opcje[i].type == INT)
				fprintf(stderr, "%d ", g_opcje[i].val);
		}
	}
	fprintf(stderr, "\n");
	return 0;
}

